
#-----------------------------------------------------------------------
# README
#-----------------------------------------------------------------------
#
# Syntax: Config_Modules [public|private] {sub-dir}
#
#-----------------------------------------------------------------------
# CHANGES 
#-----------------------------------------------------------------------
#
# 2001-02-19  Jean-Sebastien Morisset <jsmoriss@mvlan.net>
#             Moved the COMPARE_MODE section up one for-loop which
#             should improve execution speed a bit.
# 2001-02-10  Jean-Sebastien Morisset <jsmoriss@mvlan.net>
#             Initial function file written for rcf version 5.1.
#
#-----------------------------------------------------------------------
# FUNCTION
#-----------------------------------------------------------------------

Config_Modules () {
	UCASE_1="`echo $1|tr '[a-z]' '[A-Z]'`"
	LCASE_1="`echo $1|tr '[A-Z]' '[a-z]'`"
	LCASE_2="`echo $2|tr '[A-Z]' '[a-z]'`"

	case "$LCASE_2" in
		services)
			SORT_OFFSET="5"
			# Shell pattern match (not regex).
			MODULE_FILES="[0-9][0-9][0-9]-[a-z]*"
			;;
		*)	SORT_OFFSET="1"
			MODULE_FILES="*"
			;;
	esac

	# Find how far along the path string we have to sort.
	SORT_POS="`expr length ${MODULE_DIR}/${LCASE_1}/${LCASE_2}/`"
	SORT_POS="`expr $SORT_POS + $SORT_OFFSET`"

	# Include each module in the configuration file. We sort the list to
	# keep the options sorted within the configuration file. i.e. ssh
	# will be listed before telnet, etc.
	#
	for FILE in `find ${MODULE_DIR}/${LCASE_1}/${LCASE_2} \
		-name "${MODULE_FILES}" -type f -follow -maxdepth 1 \
			| sort -b -f -k 1.${SORT_POS}`
	do
		# Just in case...
		[ -f "$FILE" ] || continue

		# Gather module configuration.
		       NAME="`sed -n 's/^#[nN]#  *\([A-Za-z0-9]*\)$/\1/p'  $FILE|tr '[A-Z]' '[a-z]'`"
		    ACTIONS="`sed -n 's/^#[aA]#  *\([A-Za-z0-9 ]*\)$/\1/p' $FILE|tr '[A-Z]' '[a-z]'`"
		       TYPE="`sed -n 's/^#[tT]#  *\([A-Za-z0-9]*\)$/\1/p'  $FILE|tr '[A-Z]' '[a-z]'`"
		MODULE_MODE="`sed -n 's/^#[mM]#  *\([ 0-9]*\)/\1/p' $FILE`"

		# Skip this module if we don't have all the essential config values.
		[ "$NAME" -a "$ACTIONS" -a "$TYPE" ] || continue
	
		# Determine if this module supports d/mz clusters.
	    [ "`grep -E '^#[iI]#.* cluster( |\$)' $FILE|wc -l`" -gt "0" ] \
			&& CLUSTER_MODULE="yes" || CLUSTER_MODULE="no"

		# Pick-up all the interface names we'll be using for the options.
		eval INTERFACES=\"\$${UCASE_1}_INTERFACES\"

		# Add d/mz interfaces names (if any).
		[ "$LCASE_1" = "public" ] && INTERFACES="$INTERFACES $DMZ_INTERFACES"
		[ "$LCASE_1" = "private" ] && INTERFACES="$INTERFACES $MZ_INTERFACES"

		# Create a temporary file to store the options.
		> ${CONF}.options.tmp
		chmod 600 ${CONF}.options.tmp

		# This variable will let us keep all the options flushed right.
		MAX_OPTION_LEN="0"

		# For each public/private dmz/mz interface, start building the options.
		for INTERFACE in $INTERFACES
		do
			UCASE_INTERFACE="`echo $INTERFACE|tr '[a-z]' '[A-Z]'|tr ':-' '_'`"

			# Check to see if cluster names have been defined for this interface.
			[ "$LCASE_1" = "public" ] && eval CLUSTER_NAMES=\"\$DMZ_${UCASE_INTERFACE}_CLUSTER_NAMES\"
			[ "$LCASE_1" = "private" ] && eval CLUSTER_NAMES=\"\$MZ_${UCASE_INTERFACE}_CLUSTER_NAMES\"

			# Construct an array for the interface section of the option name.
			INTERFACE_FIELDS="$INTERFACE"

			# Add any clusters found to the array. An example would be "eth0:allservers".
			for CLUSTER_NAME in $CLUSTER_NAMES
			do
				INTERFACE_FIELDS="$INTERFACE_FIELDS ${INTERFACE}:${CLUSTER_NAME}"
			done

			for INTERFACE_FIELD in $INTERFACE_FIELDS
			do
				# For regular interfaces, the INTERFACE_FIELDS variable contains only
				# a single interface name. For DMZs, the interface has been joined with
				# cluster names (i.e. "eth3:webservers eth0:1:ftpservers", etc.).
				CLUSTER_MATCH="`echo \"$INTERFACE_FIELD\"|grep \"^${INTERFACE}:.*\$\"`"

				COMPARE_MODE=""
				if [ "$CLUSTER_MATCH" ]
				then
					# Skip any module which doesn't support clusters
					[ "$CLUSTER_MODULE" = "no" ] && continue
				
					case $LCASE_1 in
						public)		COMPARE_MODE="$DMZ_CLUSTERS_MODE" ;;
						private)	COMPARE_MODE="$MZ_CLUSTERS_MODE" ;;
					esac
				else
					if [ "$CLUSTER_NAMES" ]
					then
						case $LCASE_1 in
							public)		COMPARE_MODE="$DMZ_INTERFACES_MODE" ;;
							private)	COMPARE_MODE="$MZ_INTERFACES_MODE" ;;
						esac
					else
						case $LCASE_1 in
							public)		COMPARE_MODE="$PUBLIC_INTERFACES_MODE" ;;
							private)	COMPARE_MODE="$PRIVATE_INTERFACES_MODE" ;;
						esac
					fi
				fi

				[ "$MODULE_MODE" -a "`echo $MODULE_MODE|grep \"$COMPARE_MODE\"|wc -l`" -eq "0" ] \
					&& continue

				# For each defined action (accept, deny, ignore, etc.) construct the option.
				for ACTION in $ACTIONS
				do
					# Exclude forwarding modules for clusters
					[ "$CLUSTER_MATCH" -a "$ACTION" = "forward" ] && continue

		  			DEFAULT_VALUE="`sed -n \"s/^#[vV]#  *${ACTION}  *\(.*\)\$/\1/p\" $FILE`"
					VAR="`echo ${ACTION}-${INTERFACE_FIELD}-${NAME}-${TYPE}|tr '[A-Z]' '[a-z]'|tr '_' '-'`"
					REC="`echo ${ACTION}_${INTERFACE_FIELD}_${NAME}_${TYPE}|tr '[a-z]' '[A-Z]'|tr ':-' '_'`"
					
					[ ! "`Defined $REC`" ] && eval : \${$REC:=\"$DEFAULT_VALUE\"}
					
					OPTION_LEN="`expr length $VAR`"
					[ "$OPTION_LEN" -gt "$MAX_OPTION_LEN" ] && MAX_OPTION_LEN="$OPTION_LEN"

					eval echo "${VAR}=\$${REC}" >> ${CONF}.options.tmp

				done
			done
			unset INTERFACE_FIELDS INTERFACE_FIELD COMPARE_MODE
		done
		unset DEFAULT_VALUE INTERFACES INTERFACE
		unset CLUSTER_NAMES CLUSTER_NAME ACTION VAR REC

		# Make sure we have some options before printing the description.
		if [ "$MAX_OPTION_LEN" -gt "0" ]
		then
	  		sed -n 's/^#[dD]#\(.*\)$/#\1/p' $FILE >> "${CONF}.tmp"
			[ "`grep '^#[dD]#' $FILE|wc -l`" -gt 0 ] && echo "#" >> "${CONF}.tmp"

			gawk -v LEN="$MAX_OPTION_LEN" 'BEGIN { FS="="; }
				{ printf("%" LEN "s = %s\n", $1, $2); }
				END { print ""; }' ${CONF}.options.tmp >> "${CONF}.tmp"
		fi

		rm -f ${CONF}.options.tmp
		unset MAX_OPTION_LEN
		unset NAME ACTIONS TYPE MODULE_MODE

		TOTAL_MODULES="`expr $TOTAL_MODULES + 1`"
		echo -n "."
	done
	unset UCASE_1 LCASE_1 LCASE_2 MODULE_FILES LIMIT_TYPE SORT_POS
}

