#!/bin/sh

umask 077

APPNAME="rcf"
VERSION="5.1"
PREFIX=""

# Make sure we have all the standard OS paths available. I also prefer
# using the standard OS utilities instead of what we might find under 
# /usr/local/bin. :-)
#
ORIGINAL_PATH="$PATH"
PATH="/sbin:/usr/sbin:/bin:/usr/bin:$PATH"
export PATH

Mkdir () {
	for DIR in $*
	do
		if [ ! -d $DIR ]
		then
			echo "Creating ${DIR}..."
			mkdir -p $DIR
		fi
	done
	unset DIR
}

Backup () {
	for BACKUP in $*
	do
		if [ -e "$BACKUP" ]
		then
			echo "Moving ${BACKUP} to ${BACKUP}.${DATE}..."
			mv ${BACKUP} ${BACKUP}.${DATE}
		fi
	done
}

Security_Check () {
	echo ""
	echo "Performing Security Checks..."

	# Before calling this function, the PATH was prefixed with the standard
	# binary directories. This minimizes the chance of finding a bad egrep
	# or sed which are used here.
	#
	if [ "`echo $PATH|egrep -c ':\.(:|/|$)'`" -gt 0 ]
	then
		FAILED="yes"
		cat <<EOF

Your PATH variable contains relative directories (doesn't start with '/').
For security reasons, you should remove these directories from your PATH
permanently.
EOF
		echo ""
		[ "$ORIGINAL_PATH" ] && echo "Original PATH=${PATH}"
		PATH="`echo $PATH|sed 's/:[^\/][^:]*:*/:/g'|sed 's/:$//'`"
		export PATH
		echo "   Fixed PATH=${PATH}"
	fi

	# Now that we've potentially broken the PATH variable <smile>, make
	# sure we can still get at all the binaries we'll need.
	#
	for BIN in cp chkconfig egrep install mv tar
	do
		if [ ! "`which $BIN 2>/dev/null`" ]
		then
			[ ! "$FAILED" ] && echo ""; FAILED="yes"
			echo "Unable to locate $BIN in any PATH directory!"
		fi
	done

	if [ "$FAILED" ]
	then
		echo ""
		echo "These problems must be addressed before running `basename $0` again."
		echo ""
		exit 1
	fi
}

# ======================================================================
# MAIN
# ======================================================================

Security_Check

DATE="`date +'%y%m%d%H%M'`"

#
# backup previous installations
#
echo ""
echo "Backing up previous installations..."
echo ""
Backup \
	${PREFIX}/etc/rc.d/rc.firewall \
	${PREFIX}/etc/rc.d/init.d/firewall \
	${PREFIX}/etc/firewall-modules \
	${PREFIX}/etc/firewall

if [ -f /etc/firewall.conf ]
then
	echo "Copying ${PREFIX}/etc/firewall.conf to ${PREFIX}/etc/firewall.conf.${DATE}..."
	cp ${PREFIX}/etc/firewall.conf ${PREFIX}/etc/firewall.conf.${DATE}
fi

#
# create any missing directories.
#
echo ""
echo "Checking for missing directories..."
echo ""
Mkdir \
	${PREFIX} \
	${PREFIX}/usr/man \
	${PREFIX}/etc/firewall \
	${PREFIX}/etc/firewall/functions \
	${PREFIX}/etc/firewall/groups \
	${PREFIX}/etc/firewall/modules \
	${PREFIX}/etc/firewall/sbin \
	${PREFIX}/etc/rc.d/init.d \
	${PREFIX}/usr/doc/${APPNAME}-${VERSION}

for num in 0 1 2 3 4 5 6; do Mkdir ${PREFIX}/etc/rc.d/rc${num}.d; done

#
# install scripts
#
echo ""
echo "Installing firewall scripts..."
echo ""
install -o root -g root -m 0700 firewall ${PREFIX}/etc/rc.d/init.d/firewall
install -o root -g root -m 0700 rcf ${PREFIX}/etc/firewall/sbin/rcf

ln -s /etc/firewall/sbin/rcf /etc/rc.d/rc.firewall

#
# extract archives
#
echo ""
echo "Extracting firewall functions to ${PREFIX}/etc/firewall/functions..."
echo ""
tar -xvp -C ${PREFIX}/etc/firewall -f functions.tar

echo ""
echo "Extracting firewall groups to ${PREFIX}/etc/firewall/groups..."
echo ""
tar -xvp -C ${PREFIX}/etc/firewall -f groups.tar

echo ""
echo "Extracting firewall modules to ${PREFIX}/etc/firewall/modules..."
echo ""
tar -xvp -C ${PREFIX}/etc/firewall -f modules.tar

echo ""
echo "Extracting man pages to ${PREFIX}/usr/man..."
echo ""
tar -xvp -C ${PREFIX}/usr/man -f man.tar

#
# install documentation
#
echo ""
echo "Installing documentation under ${PREFIX}/usr/doc/${APPNAME}-${VERSION}..."
echo ""
cp ChangeLog FAQ.html INSTALL LICENSE ${PREFIX}/usr/doc/${APPNAME}-${VERSION}

echo ""
echo "Creating ${PREFIX}/etc/firewall.conf configuration file..."
echo ""
${PREFIX}/etc/firewall/sbin/rcf --config ${PREFIX}/etc/firewall.conf --update-config

#
# add firewall to startup directories
#
echo ""
echo "Configuring firewall startup scripts..."
echo ""
chkconfig --add firewall

